require "lighthouse/core/version"
require "lighthouse/client"

class LighthouseException < Exception
end

module Lighthouse
  STATUS_MAPPER = {
    "g" => "good",
    "p" => "good",
    "pass" => "good",
    "passed" => "good",
    "true" => "good",
    "good" => "good",
    "t" => "good",
    "1" => "good",
    "nil" => "medium",
    "medium" => "medium",
    "med" => "medium",
    "normal" => "medium",
    "middle" => "medium",
    "fail" => "bad",
    "failed" => "bad",
    "failure" => "bad",
    "bad" => "bad",
    "false" => "bad",
    "0" => "bad",
    "b" => "bad",
    "f" => "bad"
  }

  class << self
    def project(name="unknown")
      raise "project shouldn't be inside project block." if @project != nil
      @project=name
      if block_given?
        yield
      end
      rescue => e
        raise "This is custom raising error\nChange this at Lighthouse::Core\n"+e.message
      ensure
      @node=nil
      @topic=nil
      @group=nil
      @project=nil
    end

    def topic(name="unknown", options={})
      raise "topic shouldn't be inside topic block." if @topic != nil
      @topic=name
      if block_given?
        yield
      end
      ensure
      @topic=nil
    end

    def node(name="unknown", options={})
      raise "node shouldn't be inside node block." if @node != nil
      @node=name
      @group=options[:group]
      @contact=options[:contact]
      @variables=options[:variables]
      if block_given?
        yield
      end
      rescue LighthouseException
      rescue => e
        report(:fail, e.message, true)
      ensure
      @node=nil
      @group=nil
      @contact=nil
      @variables=nil
    end

    def report(status, message=nil, noraise=false)
      status = map_status(status)
      Client.send_message({:status => status, :message => message, :project => @project, :topic => @topic, :group => @group, :node => @node, :contact => @contact, :variables => @variables})
      raise LighthouseException.new("done") unless noraise
    end

    def map_status(status)
      STATUS_MAPPER[(status.to_s rescue nil)]
    end
  end
end
